@@ -1,8 +1,9 @@
 # pylint: disable=no-member, no-self-use, redefined-outer-name
 # cspell:ignore atol doprint
+from __future__ import annotations
+
 import inspect
 import textwrap
-from typing import Dict, List, Tuple
 
 import numpy as np
 import pytest
@@ -38,6 +39,7 @@ from ampform.kinematics import (
     create_four_momentum_symbols,
     three_momentum_norm,
 )
+from ampform.sympy import NumPyPrintable
 from ampform.sympy._array_expressions import (
     ArrayMultiplication,
     ArraySlice,
@@ -47,8 +49,8 @@ from ampform.sympy._array_expressions import (
 
 @pytest.fixture(scope="session")
 def topology_and_momentum_symbols(
-    data_sample: Dict[int, np.ndarray]
-) -> Tuple[Topology, FourMomenta]:
+    data_sample: dict[int, np.ndarray]
+) -> tuple[Topology, FourMomenta]:
     n = len(data_sample)
     assert n == 4
     topologies = create_isobar_topologies(n)
@@ -59,15 +61,15 @@ def topology_and_momentum_symbols(
 
 @pytest.fixture(scope="session")
 def helicity_angles(
-    topology_and_momentum_symbols: Tuple[Topology, FourMomenta]
-) -> Dict[str, sp.Expr]:
+    topology_and_momentum_symbols: tuple[Topology, FourMomenta]
+) -> dict[str, sp.Expr]:
     topology, momentum_symbols = topology_and_momentum_symbols
     return compute_helicity_angles(momentum_symbols, topology)
 
 
 class TestBoostMatrix:
     def test_boost_in_z_direction_reduces_to_z_boost(self):
-        p = FourMomentumSymbol("p")
+        p = FourMomentumSymbol("p", shape=[])
         expr = BoostMatrix(p)
         func = sp.lambdify(p, expr.doit(), cse=True)
         p_array = np.array([[5, 0, 0, 1]])
@@ -91,8 +93,8 @@ class TestBoostMatrix:
     def test_boost_into_rest_frame_gives_mass(
         self,
         state_id: int,
-        data_sample: Dict[int, np.ndarray],
-        topology_and_momentum_symbols: Tuple[Topology, FourMomenta],
+        data_sample: dict[int, np.ndarray],
+        topology_and_momentum_symbols: tuple[Topology, FourMomenta],
     ):
         # pylint: disable=too-many-locals
         pi0_mass = 0.135
@@ -113,9 +115,9 @@ class TestBoostMatrix:
 
     @pytest.mark.parametrize("state_id", [0, 2, 3])
     def test_boosting_back_gives_original_momentum(
-        self, state_id: int, data_sample: Dict[int, np.ndarray]
+        self, state_id: int, data_sample: dict[int, np.ndarray]
     ):
-        p = FourMomentumSymbol("p")
+        p = FourMomentumSymbol("p", shape=[])
         boost = BoostMatrix(p)
         inverse_boost = BoostMatrix(NegativeMomentum(p))
         expr = ArrayMultiplication(inverse_boost, boost, p)
@@ -128,7 +130,7 @@ class TestBoostMatrix:
 
 class TestBoostZMatrix:
     def test_boost_into_own_rest_frame_gives_mass(self):
-        p = FourMomentumSymbol("p")
+        p = FourMomentumSymbol("p", shape=[])
         n_events = _ArraySize(p)
         beta = three_momentum_norm(p) / Energy(p)
         expr = BoostZMatrix(beta, n_events)
@@ -139,8 +141,8 @@ class TestBoostZMatrix:
         mass = 4.89897949
         assert pytest.approx(boosted_array[0]) == [mass, 0, 0, 0]
 
-        expr = InvariantMass(p)
-        func = sp.lambdify(p, expr.doit(), cse=True)
+        mass_expr = InvariantMass(p)
+        func = sp.lambdify(p, mass_expr.doit(), cse=True)
         mass_array = func(p_array)
         assert pytest.approx(mass_array[0]) == mass
 
@@ -190,10 +192,10 @@ class TestBoostZMatrix:
 class TestFourMomentumXYZ:
     def symbols(
         self,
-    ) -> Tuple[
+    ) -> tuple[
         FourMomentumSymbol, Energy, FourMomentumX, FourMomentumY, FourMomentumZ
     ]:
-        p = FourMomentumSymbol("p")
+        p = FourMomentumSymbol("p", shape=[])
         e = Energy(p)
         p_x = FourMomentumX(p)
         p_y = FourMomentumY(p)
@@ -213,7 +215,8 @@ class TestFourMomentumXYZ:
         assert sp.latex(p_x) == "{p}_x"
         assert sp.latex(p_y) == "{p}_y"
         assert sp.latex(p_z) == "{p}_z"
-        a, b = sp.symbols("A B", cls=ArraySymbol)
+        a = ArraySymbol("A", shape=[])
+        b = ArraySymbol("B", shape=[])
         expr = FourMomentumX(a + b)
         assert sp.latex(expr) == R"\left(A + B\right)_x"
 
@@ -230,11 +233,11 @@ class TestInvariantMass:
     )
     def test_numpy(
         self,
-        data_sample: Dict[int, np.ndarray],
+        data_sample: dict[int, np.ndarray],
         state_id: int,
         expected_mass: float,
     ):
-        p = FourMomentumSymbol(f"p{state_id}")
+        p = FourMomentumSymbol(f"p{state_id}", shape=[])
         mass = InvariantMass(p)
         np_mass = sp.lambdify(p, mass.doit(), cse=True)
         four_momenta = data_sample[state_id]
@@ -246,7 +249,7 @@ class TestInvariantMass:
 class TestThreeMomentum:
     @property
     def p_norm(self) -> ThreeMomentum:
-        p = FourMomentumSymbol("p")
+        p = FourMomentumSymbol("p", shape=[])
         return ThreeMomentum(p)
 
     def test_latex(self):
@@ -260,8 +263,8 @@ class TestThreeMomentum:
 
 class TestPhi:
     @property
-    def phi(self) -> Theta:
-        p = FourMomentumSymbol("p")
+    def phi(self):
+        p = FourMomentumSymbol("p", shape=[])
         return Phi(p)
 
     def test_latex(self):
@@ -276,8 +279,8 @@ class TestPhi:
 
 class TestTheta:
     @property
-    def theta(self) -> Theta:
-        p = FourMomentumSymbol("p")
+    def theta(self):
+        p = FourMomentumSymbol("p", shape=[])
         return Theta(p)
 
     def test_latex(self):
@@ -294,8 +297,8 @@ class TestTheta:
 
 
 class TestNegativeMomentum:
-    def test_same_as_inverse(self, data_sample: Dict[int, np.ndarray]):
-        p = FourMomentumSymbol("p")
+    def test_same_as_inverse(self, data_sample: dict[int, np.ndarray]):
+        p = FourMomentumSymbol("p", shape=[])
         expr = NegativeMomentum(p)
         func = sp.lambdify(p, expr.doit(), cse=True)
         for p_array in data_sample.values():
@@ -413,7 +416,7 @@ class TestOnesZerosArray:
     @pytest.mark.parametrize("shape", [10, (4, 2), [3, 5, 7]])
     def test_numpycode(self, array_type, shape):
         if array_type == "ones":
-            expr_class = _OnesArray
+            expr_class: type[NumPyPrintable] = _OnesArray
             array_func = np.ones
         elif array_type == "zeros":
             expr_class = _ZerosArray
@@ -536,11 +539,11 @@ class TestOnesZerosArray:
 )
 def test_compute_helicity_angles(  # pylint: disable=too-many-arguments
     use_cse: bool,
-    data_sample: Dict[int, np.ndarray],
-    topology_and_momentum_symbols: Tuple[Topology, FourMomenta],
+    data_sample: dict[int, np.ndarray],
+    topology_and_momentum_symbols: tuple[Topology, FourMomenta],
     angle_name: str,
     expected_values: np.ndarray,
-    helicity_angles: Dict[str, sp.Expr],
+    helicity_angles: dict[str, sp.Expr],
 ):
     _, momentum_symbols = topology_and_momentum_symbols
     four_momenta = data_sample.values()
@@ -551,7 +554,7 @@ def test_compute_helicity_angles(  # pylint: disable=too-many-arguments
 
 
 def test_compute_invariant_masses_names(
-    topology_and_momentum_symbols: Tuple[Topology, FourMomenta]
+    topology_and_momentum_symbols: tuple[Topology, FourMomenta]
 ):
     topology, momentum_symbols = topology_and_momentum_symbols
     invariant_masses = compute_invariant_masses(momentum_symbols, topology)
@@ -567,8 +570,8 @@ def test_compute_invariant_masses_names(
 
 
 def test_compute_invariant_masses_single_mass(
-    data_sample: Dict[int, np.ndarray],
-    topology_and_momentum_symbols: Tuple[Topology, FourMomenta],
+    data_sample: dict[int, np.ndarray],
+    topology_and_momentum_symbols: tuple[Topology, FourMomenta],
 ):
     topology, momentum_symbols = topology_and_momentum_symbols
     momentum_values = data_sample.values()
@@ -584,8 +587,8 @@ def test_compute_invariant_masses_single_mass(
 @pytest.mark.parametrize("mass_name", ["m_23", "m_123", "m_0123"])
 def test_compute_invariant_masses(
     mass_name: str,
-    data_sample: Dict[int, np.ndarray],
-    topology_and_momentum_symbols: Tuple[Topology, FourMomenta],
+    data_sample: dict[int, np.ndarray],
+    topology_and_momentum_symbols: tuple[Topology, FourMomenta],
 ):
     topology, momentum_symbols = topology_and_momentum_symbols
     momentum_values = data_sample.values()
@@ -646,8 +649,8 @@ def _generate_numpy_code(expr: sp.Expr) -> str:
 )
 def test_compute_boost_chain(
     state_id: int,
-    expected: List[str],
-    topology_and_momentum_symbols: Tuple[Topology, FourMomenta],
+    expected: list[str],
+    topology_and_momentum_symbols: tuple[Topology, FourMomenta],
 ):
     topology, momentum_symbols = topology_and_momentum_symbols
     boost_chain = compute_boost_chain(topology, momentum_symbols, state_id)
@@ -701,7 +704,7 @@ def test_compute_boost_chain(
 def test_compute_wigner_rotation_matrix(
     state_id: int,
     expected: str,
-    topology_and_momentum_symbols: Tuple[Topology, FourMomenta],
+    topology_and_momentum_symbols: tuple[Topology, FourMomenta],
 ):
     topology, momenta = topology_and_momentum_symbols
     expr = compute_wigner_rotation_matrix(topology, momenta, state_id)
@@ -718,8 +721,8 @@ def test_compute_wigner_rotation_matrix(
 )
 def test_compute_wigner_rotation_matrix_numpy(
     state_id: int,
-    data_sample: Dict[int, np.ndarray],
-    topology_and_momentum_symbols: Tuple[Topology, FourMomenta],
+    data_sample: dict[int, np.ndarray],
+    topology_and_momentum_symbols: tuple[Topology, FourMomenta],
 ):
     topology, momenta = topology_and_momentum_symbols
     expr = compute_wigner_rotation_matrix(topology, momenta, state_id)
